//=======================================================================
//                         USB Target CDC sample
//
// File:
//   usbs_base.c
//
// Abstract:
//   OpenCores USB IP Core control functions
//
//  Created:
//    27 November 2014
//======================================================================
//
// Copyright (c) 2014, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Revision History
// $Date: 2015-01-09 08:44:55 +0900 (Fri, 09 Jan 2015) $
// $Rev: 16 $

#include <system.h>
#include <altera_avalon_pio_regs.h>
#include "usbs_base.h"

// device state

st_usbs_env g_usbs_env;

unsigned char g_set_line_coding_buf[] = {
  0x80,0x25,0x00,0x00,0x00,0x00,0x08
};

void setLED(int n) {
  IOWR_ALTERA_AVALON_PIO_DATA(PIO_1_BASE,1<<n);
}
void set7SEG(char n) {
   //            bit pos
   //     -        0
   //    | |      5 1
   //     -        6
   //    | |      4 2
   //     -        3
  switch (n) {
  case '0':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x3f);
    break;
  case '1':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x06);
    break;
  case '2':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x5b);
    break;
  case '3':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x4f);
    break;
  case '4':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x66);
    break;
  case '5':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x6d);
    break;
  case '6':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x7d);
    break;
  case '7':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x27);
    break;
  case '8':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x7f);
    break;
  case '9':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x67);
    break;
  case 'A':
  case 'a':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x77);
    break;
  case 'B':
  case 'b':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x7c);
    break;
  case 'C':
  case 'c':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x58);
    break;
  case 'D':
  case 'd':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x5e);
    break;
  case 'E':
  case 'e':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x79);
    break;
  case 'F':
  case 'f':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x71);
    break;
  case 'G':
  case 'g':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x3d);
    break;
  case 'H':
  case 'h':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x74);
    break;
  case 'I':
  case 'i':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x04);
    break;
  case 'J':
  case 'j':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x1e);
    break;
  case 'K':
  case 'k':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x75);
    break;
  case 'L':
  case 'l':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x38);
    break;
  case 'M':
  case 'm':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x37);
    break;
  case 'N':
  case 'n':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x54);
    break;
  case 'O':
  case 'o':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x5c);
    break;
  case 'P':
  case 'p':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x73);
    break;
  case 'Q':
  case 'q':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x7b);
    break;
  case 'R':
  case 'r':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x50);
    break;
  case 'S':
  case 's':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x6c);
    break;
  case 'T':
  case 't':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x78);
    break;
  case 'U':
  case 'u':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x1c);
    break;
  case 'V':
  case 'v':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x3e);
    break;
  case 'W':
  case 'w':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x7e);
    break;
  case 'X':
  case 'x':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x76);
    break;
  case 'Y':
  case 'y':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x6e);
    break;
  case 'Z':
  case 'z':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x1b);
    break;
  default:
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x80);
    break;
  }
}

void endpoint0_trans() {
  int i;
  int len;
  int stat;
  g_usbs_env.EP0.active = 0;
  stat = USB_ENDPOINT0_TRANSTYPE_STATUS_REG;
  if (stat == 0) {
    control_transfer_setup_trans();
  } else
  if (stat == 1) {
    // IN
    control_transfer_in_trans();  // set  next data
  } else
  if (stat == 2) {
    // OUT
    len = rx_fifo_cnt(0);
    control_transfer_out_trans(len);  // set  next data
  }
}

void endpoint2_trans() {
  g_usbs_env.EP2.active = 0;
  update_pid_ep(2);
  // BULK-IN transaction
  // for next OUT data
  set_control_reg_ep(1);
}

void set_control_reg_ep(int en) {
  switch (en) {
  case 0:
    g_usbs_env.EP0.active = 1;
    USB_ENDPOINT0_CONTROL_REG = 0x3 | (g_usbs_env.EP0.out_data_type << 2);
    break;
  case 1:
    g_usbs_env.EP1.active = 1;
    USB_ENDPOINT1_CONTROL_REG = 0x3 | (g_usbs_env.EP1.out_data_type << 2);
    break;
  case 2:
    g_usbs_env.EP2.active = 1;
    USB_ENDPOINT2_CONTROL_REG = 0x3 | (g_usbs_env.EP2.out_data_type << 2);
    break;
  case 3:
    g_usbs_env.EP3.active = 1;
    USB_ENDPOINT3_CONTROL_REG = 0x3 | (g_usbs_env.EP3.out_data_type << 2);
    break;
  }
}

void set_control_reg_stall_ep(int en) {
  switch (en) {
  case 0:
    g_usbs_env.EP0.active = 1;
    USB_ENDPOINT0_CONTROL_REG = 0xb;
    set7SEG('8');
    break;
  case 1:
    g_usbs_env.EP1.active = 1;
    USB_ENDPOINT1_CONTROL_REG = 0xb;
    break;
  case 2:
    g_usbs_env.EP2.active = 1;
    USB_ENDPOINT2_CONTROL_REG = 0xb;
    break;
  case 3:
    g_usbs_env.EP3.active = 1;
    USB_ENDPOINT3_CONTROL_REG = 0xb;
    break;
  }
}

void update_pid_ep(int en) {
  switch (en) {
  case 0:
    if (g_usbs_env.EP0.out_data_type == 0) g_usbs_env.EP0.out_data_type = 1;
    else g_usbs_env.EP0.out_data_type = 0;
    break;
  case 1:
    if (g_usbs_env.EP1.out_data_type == 0) g_usbs_env.EP1.out_data_type = 1;
    else g_usbs_env.EP1.out_data_type = 0;
    break;
  case 2:
    if (g_usbs_env.EP2.out_data_type == 0) g_usbs_env.EP2.out_data_type = 1;
    else g_usbs_env.EP2.out_data_type = 0;
    break;
  case 3:
    if (g_usbs_env.EP3.out_data_type == 0) g_usbs_env.EP3.out_data_type = 1;
    else g_usbs_env.EP3.out_data_type = 0;
    break;
  }
}

void set_control_reg_pid0_ep(int en) {
  switch (en) {
  case 0:
    g_usbs_env.EP0.active = 1;
    USB_ENDPOINT0_CONTROL_REG = 0x3;
    break;
  case 1:
    g_usbs_env.EP1.active = 1;
    USB_ENDPOINT1_CONTROL_REG = 0x3;
    break;
  case 2:
    g_usbs_env.EP2.active = 1;
    USB_ENDPOINT2_CONTROL_REG = 0x3;
    break;
  case 3:
    g_usbs_env.EP3.active = 1;
    USB_ENDPOINT3_CONTROL_REG = 0x3;
    break;
  }
}

void set_control_reg_pid1_ep(int en) {
  switch (en) {
  case 0:
    g_usbs_env.EP0.active = 1;
    USB_ENDPOINT0_CONTROL_REG = 0x3|0x4;
    break;
  case 1:
    g_usbs_env.EP1.active = 1;
    USB_ENDPOINT1_CONTROL_REG = 0x3|0x4;
    break;
  case 2:
    g_usbs_env.EP2.active = 1;
    USB_ENDPOINT2_CONTROL_REG = 0x3|0x4;
    break;
  case 3:
    g_usbs_env.EP3.active = 1;
    USB_ENDPOINT3_CONTROL_REG = 0x3|0x4;
    break;
  }
}

void endpoint1_trans() {
   // BULK-OUT transaction
   unsigned char buf[64];
   int len = 0;
   int i;
   g_usbs_env.EP1.active = 0;
   update_pid_ep(1);
   len = rx_fifo_cnt(1);
   for (i=0;i<len;i++)
     buf[i] = USB_EP1_RX_FIFO_DATA;
   if (buf[0] == MESSAGE_KEY)
     disp_terminal_message();
   else {
     set_control_reg_ep(2);
     // EP2 setup
     for (i=0;i<len;i++)
       USB_EP2_TX_FIFO_DATA = buf[i];
     set7SEG(buf[0]);
   }
}

void endpoint3_trans() {
}

void disp_terminal_message() {
  // EP2 setup
  USB_EP2_TX_FIFO_CONTROL_REG = 1;
  USB_EP2_TX_FIFO_DATA = 'O';  // IN
  USB_EP2_TX_FIFO_DATA = 'p';  // IN
  USB_EP2_TX_FIFO_DATA = 'e';  // IN
  USB_EP2_TX_FIFO_DATA = 'n';  // IN
  USB_EP2_TX_FIFO_DATA = 'C';  // IN
  USB_EP2_TX_FIFO_DATA = 'o';  // IN
  USB_EP2_TX_FIFO_DATA = 'r';  // IN
  USB_EP2_TX_FIFO_DATA = 'e';  // IN
  USB_EP2_TX_FIFO_DATA = 's';  // IN
  USB_EP2_TX_FIFO_DATA = ' ';  // IN
  USB_EP2_TX_FIFO_DATA = 'U';  // IN
  USB_EP2_TX_FIFO_DATA = 'S';  // IN
  USB_EP2_TX_FIFO_DATA = 'B';  // IN
  USB_EP2_TX_FIFO_DATA = ' ';  // IN
  USB_EP2_TX_FIFO_DATA = 'I';  // IN
  USB_EP2_TX_FIFO_DATA = 'P';  // IN
  USB_EP2_TX_FIFO_DATA = ' ';  // IN
  USB_EP2_TX_FIFO_DATA = 'C';  // IN
  USB_EP2_TX_FIFO_DATA = 'o';  // IN
  USB_EP2_TX_FIFO_DATA = 'r';  // IN
  USB_EP2_TX_FIFO_DATA = 'e';  // IN
  USB_EP2_TX_FIFO_DATA = ' ';  // IN
  USB_EP2_TX_FIFO_DATA = 'C';  // IN
  USB_EP2_TX_FIFO_DATA = 'D';  // IN
  USB_EP2_TX_FIFO_DATA = 'C';  // IN
  USB_EP2_TX_FIFO_DATA = ' ';  // IN
  USB_EP2_TX_FIFO_DATA = 'A';  // IN
  USB_EP2_TX_FIFO_DATA = 'C';  // IN
  USB_EP2_TX_FIFO_DATA = 'M';  // IN
  USB_EP2_TX_FIFO_DATA = ' ';  // IN
  USB_EP2_TX_FIFO_DATA = 'S';  // IN
  USB_EP2_TX_FIFO_DATA = 'a';  // IN
  USB_EP2_TX_FIFO_DATA = 'm';  // IN
  USB_EP2_TX_FIFO_DATA = 'p';  // IN
  USB_EP2_TX_FIFO_DATA = 'l';  // IN
  USB_EP2_TX_FIFO_DATA = 'e';  // IN
  USB_EP2_TX_FIFO_DATA = '.';  // IN
  USB_EP2_TX_FIFO_DATA = '\r';  // IN
  USB_EP2_TX_FIFO_DATA = '\n';  // IN
  set_control_reg_ep(2);
}

void init_control_cnt(int blen) {
  int n;
  n = blen/8;
  if ((blen%8)!=0) n++;
  g_usbs_env.EP0.out_data_type = 1;
  g_usbs_env.EP0.total_desc_cnt = n;
  g_usbs_env.EP0.cur_desc_cnt = 1;
  g_usbs_env.EP0.cur_byte_cnt = blen;
}


void control_transfer_setup_trans() {
  int l,i,n;
  char buf[64];
  // check request kind
  // SETUP
  l = rx_fifo_cnt(0);
  reset_stall_flag();
  for (i=0;i<l;i++)
    g_usbs_env.EP0.setup_data.PACKED[i] = USB_EP0_RX_FIFO_DATA;
  // data stage availability
  switch (g_usbs_env.EP0.setup_data.BYTE.bRequest) {
  case USR_SET_LINE_CODING:
    set_control_stage(UDC_DATA_WRITE);
    set_control_reg_pid0_ep(0);
    break;
  case USR_GET_LINE_CODING:
    set_control_stage(UDC_DATA_READ);
    l = 7;
    init_control_cnt(7);
    USB_EP0_TX_FIFO_CONTROL_REG = 1;  // clear TX FIFO
    for (i=0;i<l;i++)
      USB_EP0_TX_FIFO_DATA =   g_set_line_coding_buf[i];  // return set_coding_line data
    set_control_reg_pid1_ep(0);  // next ready for status stage: DATA1
    break;
  case USR_SET_CONTROL_LINE_STATE:
    set_control_stage(UDC_STATUS_NO_DATA);
    set_control_reg_pid1_ep(0);  // next ready for status stage: DATA1
    break;
  case USR_SET_ADDRESS:
    set_control_stage(UDC_STATUS_NO_DATA);
    set_control_reg_pid1_ep(0);  // next ready for status stage: DATA1
    break;
  case USR_GET_DESCRIPTOR:
    if (g_usbs_env.EP0.setup_data.BYTE.wValue[1] == UDT_DEVICE) {
      l = (g_usbs_env.EP0.setup_data.BYTE.wLength[1] << 8) |
      g_usbs_env.EP0.setup_data.BYTE.wLength[0];
      // num of transactions
      init_control_cnt(l);
      if (l>8) l = 8;
      USB_EP0_TX_FIFO_CONTROL_REG = 1;  // clear TX FIFO
      // set device descriptor
      for (i=0;i<l;i++)
        USB_EP0_TX_FIFO_DATA = g_usb_device_desc[i];
    } else
    if (g_usbs_env.EP0.setup_data.BYTE.wValue[1] == UDT_CONFIGURATION) {
      // return all interface,endpoint
      l = (g_usbs_env.EP0.setup_data.BYTE.wLength[1] << 8) |
           g_usbs_env.EP0.setup_data.BYTE.wLength[0];
      // num of transactions
      if (l>67) l = 67;
      init_control_cnt(l);
      USB_EP0_TX_FIFO_CONTROL_REG = 1;  // clear TX FIFO
      if (l>8) l = 8;
      for (i=0;i<l;i++)
        USB_EP0_TX_FIFO_DATA =  g_usb_conf_desc[i];
    }else
    if (g_usbs_env.EP0.setup_data.BYTE.wValue[1] == UDT_STRING) {
      l = (g_usbs_env.EP0.setup_data.BYTE.wLength[1] << 8) |
                   g_usbs_env.EP0.setup_data.BYTE.wLength[0];
      // num of transactions
      if (g_usbs_env.EP0.setup_data.BYTE.wValue[0]==0) if (l>4) l = 4;
      if (g_usbs_env.EP0.setup_data.BYTE.wValue[0]==1) if (l>40)  l = 42;
      if (g_usbs_env.EP0.setup_data.BYTE.wValue[0]==2) if (l>22) l = 22;
      if (g_usbs_env.EP0.setup_data.BYTE.wValue[0]==3) if (l>20) l = 20;
      init_control_cnt(l);
      USB_EP0_TX_FIFO_CONTROL_REG = 1;  // clear TX FIFO
      if (l>8) l=8;
      if (g_usbs_env.EP0.setup_data.BYTE.wValue[0]==0) {
        // LanguageID
        for (i=0;i<l;i++) {
          //printf("desc %d %x\n", l,g_usb_string_desc0[i]);
          USB_EP0_TX_FIFO_DATA =  g_usb_string_desc0[i];
        }
      } else
      if (g_usbs_env.EP0.setup_data.BYTE.wValue[0]==1) {
        // iManufacturer:21
        for (i=0;i<l;i++)
          USB_EP0_TX_FIFO_DATA =  g_usb_string_desc1[i];
      } else
      if (g_usbs_env.EP0.setup_data.BYTE.wValue[0]==2) {
        // iProduct:12
        for (i=0;i<l;i++)
          USB_EP0_TX_FIFO_DATA =  g_usb_string_desc2[i];
      } else
      if (g_usbs_env.EP0.setup_data.BYTE.wValue[0]==3) {
        // iSerialNumber:11
        for (i=0;i<l;i++)
          USB_EP0_TX_FIFO_DATA =  g_usb_string_desc3[i];
      } else {
      }
    }
    set_control_stage(UDC_DATA_READ);
    set_control_reg_pid1_ep(0);  // next ready for status stage: DATA1
    break;
  case USR_SET_CONFIGURATION:
    //wValue[0] contains configuration value
    set_control_stage(UDC_STATUS_NO_DATA);
    set_control_reg_pid1_ep(0);  // next ready for status stage: DATA1
    break;
  default:  // write STALL
    set_stall_flag();
    set_control_reg_stall_ep(0);
    break;
  }
}

void control_transfer_in_trans() {
  int i,l;
  char buf[64];
  if (get_stall_flag() ==1) {
    // stall process
    printf("STALL IN\n");
    set_control_reg_stall_ep(1);
  } else {
    switch (g_usbs_env.EP0.setup_data.BYTE.bRequest) {
    case USR_GET_LINE_CODING:
      set_control_stage(UDC_STATUS_READ);
      set_control_reg_pid1_ep(0);  // next ready for status stage
    break;
    case USR_GET_DESCRIPTOR:
      if (g_usbs_env.EP0.setup_data.BYTE.wValue[1] == UDT_DEVICE) {
        if (g_usbs_env.EP0.total_desc_cnt > g_usbs_env.EP0.cur_desc_cnt) {
          g_usbs_env.EP0.cur_byte_cnt -= 8;
          l = g_usbs_env.EP0.cur_byte_cnt;
          if (g_usbs_env.EP0.cur_byte_cnt>8) l =8;
          for (i=0;i<l;i++)
            USB_EP0_TX_FIFO_DATA = g_usb_device_desc[g_usbs_env.EP0.cur_desc_cnt*8+i];
          g_usbs_env.EP0.cur_desc_cnt++;
          update_pid_ep(0);
          set_control_reg_ep(0); // ready for data stage
        } else {
          set_control_stage(UDC_STATUS_READ);  // no more IN transaction, go to status stage
          set_control_reg_pid1_ep(0);  // next ready for status stage
        }
      } else
      if (g_usbs_env.EP0.setup_data.BYTE.wValue[1] == UDT_CONFIGURATION) {
        if (g_usbs_env.EP0.total_desc_cnt > g_usbs_env.EP0.cur_desc_cnt) {
          g_usbs_env.EP0.cur_byte_cnt -= 8;
          l = g_usbs_env.EP0.cur_byte_cnt;
          if (g_usbs_env.EP0.cur_byte_cnt>8) l =8;
          for (i=0;i<l;i++)
            USB_EP0_TX_FIFO_DATA =  g_usb_conf_desc[g_usbs_env.EP0.cur_desc_cnt*8+i];
          g_usbs_env.EP0.cur_desc_cnt++;
          update_pid_ep(0);
          set_control_reg_ep(0); // ready for data stage
        } else {
          set_control_stage(UDC_STATUS_READ);
          set_control_reg_pid1_ep(0);  // next ready for status stage
        }
      } else
      if (g_usbs_env.EP0.setup_data.BYTE.wValue[1] == UDT_STRING) {
        if (g_usbs_env.EP0.total_desc_cnt > g_usbs_env.EP0.cur_desc_cnt) {
          g_usbs_env.EP0.cur_byte_cnt -= 8;
          l = g_usbs_env.EP0.cur_byte_cnt;
          if (g_usbs_env.EP0.cur_byte_cnt>8) l =8;
          for (i=0;i<l;i++) {
            if (g_usbs_env.EP0.setup_data.BYTE.wValue[0]==0) {
              // LanguageID
              USB_EP0_TX_FIFO_DATA =  g_usb_string_desc0[g_usbs_env.EP0.cur_desc_cnt*8+i];
            } else
            if (g_usbs_env.EP0.setup_data.BYTE.wValue[0]==1) {
              // iManufacturer:21
              USB_EP0_TX_FIFO_DATA =  g_usb_string_desc1[g_usbs_env.EP0.cur_desc_cnt*8+i];
            } else
            if (g_usbs_env.EP0.setup_data.BYTE.wValue[0]==2) {
              // iProduct:12
              USB_EP0_TX_FIFO_DATA =  g_usb_string_desc2[g_usbs_env.EP0.cur_desc_cnt*8+i];
            } else
            if (g_usbs_env.EP0.setup_data.BYTE.wValue[0]==3) {
              // iSerialNumber:11
              USB_EP0_TX_FIFO_DATA =  g_usb_string_desc3[g_usbs_env.EP0.cur_desc_cnt*8+i];
            }
          }
          g_usbs_env.EP0.cur_desc_cnt++;
          update_pid_ep(0);
          set_control_reg_ep(0); // ready for data stage
        } else {
          set_control_stage(UDC_STATUS_READ);
          set_control_reg_pid1_ep(0);  // next ready for status stage
        }
      }
      break;
    case USR_SET_LINE_CODING:
      //disp_terminal_message();  // send initial message to terminal
      set_control_stage(UDC_IDLE);
      set_control_reg_pid0_ep(0);  // next ready for status stage
    break;
    case USR_SET_ADDRESS:
      // status stage returns zero data
      // the device does not change its device address until
      // after the Status stage of this request
      USB_SC_ADDRESS = g_usbs_env.EP0.setup_data.BYTE.wValue[0];
      g_usbs_env.usb_device_state = UDS_ADDRESS;
      setLED(1);

      set_control_stage(UDC_IDLE);
      USB_EP0_TX_FIFO_CONTROL_REG = 1;  // clear TX FIFO
      set_control_reg_pid0_ep(0);  // next ready for status stage
      break;
    case USR_SET_CONFIGURATION:
      g_usbs_env.usb_device_state = UDS_CONFIGURED;
      setLED(2);
      set_control_stage(UDC_IDLE);
      USB_EP0_TX_FIFO_CONTROL_REG = 1;  // clear TX FIFO
      set_control_reg_pid0_ep(0);  // next ready for status stage
      set_control_reg_ep(1);  // EP1 OUT activation (it's not nesesariliy)
      USB_ENDPOINT2_CONTROL_REG = 0x1;
      USB_ENDPOINT3_CONTROL_REG = 0x1;
      break;
    case USR_SET_CONTROL_LINE_STATE:
      set_control_stage(UDC_IDLE);
      USB_EP0_TX_FIFO_CONTROL_REG = 1;  // clear TX FIFO
      set_control_reg_pid0_ep(0);  // next ready for status stage
      //disp_terminal_message();  // send initial message
      break;
    default:
      set_control_stage(UDC_IDLE);
      set_control_reg_pid0_ep(0);  // next ready for status stage
      break;
    }
  }  // normal process
}

void control_transfer_out_trans() {
  int i;

  if (get_stall_flag() ==1) {
    // stall process
    set_control_reg_stall_ep(1);
  } else {
    switch (g_usbs_env.EP0.setup_data.BYTE.bRequest) {
      case USR_SET_LINE_CODING:
        for (i=0;i<7;i++)
          g_set_line_coding_buf[i] = USB_EP0_RX_FIFO_DATA;
          set_control_stage(UDC_STATUS_WRITE);
          set_control_reg_pid1_ep(0);  // next ready for status stage: DATA1
          break;
      case USR_GET_DESCRIPTOR:
      case USR_GET_LINE_CODING:
        USB_EP0_TX_FIFO_CONTROL_REG = 1;
        set_control_reg_pid0_ep(0);  // next ready for status stage
        set_control_stage(UDC_IDLE);
        break;
    }
  }  // normal process
}

void show_status() {
  //printf("USB_ENDPOINT0_STATUS_REG %x\n",USB_ENDPOINT0_STATUS_REG);
  //printf("USB_ENDPOINT0_TRANSTYPE_STATUS_REG %x\n",USB_ENDPOINT0_TRANSTYPE_STATUS_REG);
  //printf("USB_ENDPOINT0_NAK_TRANSTYPE_STATUS_REG %x\n",USB_ENDPOINT0_NAK_TRANSTYPE_STATUS_REG);
}

void set_control_stage(en_usbs_control_transfer_stage s) {
  g_usbs_env.EP0.control_prev_stage = g_usbs_env.EP0.control_cur_stage;
  g_usbs_env.EP0.control_cur_stage = s;
}

void usbs_init() {
  int i,j;
  USB_HOST_SLAVE_CONTROL_REG  = 0;  // SLAVE MODE
  USB_HOST_SLAVE_CONTROL_REG  = 2;  // SLAVE MODE
  USB_HOST_SLAVE_CONTROL_REG  = 0;  // wait 10 cycles
  USB_SC_CONTROL_REG = 0x71; // global enable bit
  USB_SC_INTERRUPT_MASK_REG = 0x1f;  // all enable
  reset_event_proc();
}

void reset_event_proc() {
  int i,j;
  //printf("RESET_EVENT\n");
  g_usbs_env.usb_device_state = UDS_DEFAULT;
  setLED(0);
  set_control_stage(UDC_IDLE);
  g_usbs_env.EP0.out_data_type = 0;
  g_usbs_env.EP0.cur_byte_cnt = 0;
  g_usbs_env.EP0.active = 0;
  g_usbs_env.EP0.stall_flag = 0;
  g_usbs_env.EP1.active = 0;
  g_usbs_env.EP1.out_data_type = 0;
  g_usbs_env.EP2.active = 0;
  g_usbs_env.EP2.out_data_type = 0;
  g_usbs_env.EP3.active = 0;
  g_usbs_env.EP3.out_data_type = 0;

  USB_SC_ADDRESS = 0;
  USB_EP0_RX_FIFO_CONTROL_REG = 1;
  USB_EP0_TX_FIFO_CONTROL_REG = 1;
  USB_EP1_RX_FIFO_CONTROL_REG = 1;
  USB_EP1_TX_FIFO_CONTROL_REG = 1;
  USB_EP2_RX_FIFO_CONTROL_REG = 1;
  USB_EP2_TX_FIFO_CONTROL_REG = 1;
  USB_EP3_RX_FIFO_CONTROL_REG = 1;
  USB_EP3_TX_FIFO_CONTROL_REG = 1;
  g_usbs_env.EP0.active = 1;
  USB_ENDPOINT0_CONTROL_REG = 0x3;  // enable & ready
  USB_ENDPOINT1_CONTROL_REG = 0x0;  // not enable & ready
  USB_ENDPOINT2_CONTROL_REG = 0x0;  // not enable & ready
  USB_ENDPOINT3_CONTROL_REG = 0x0;  // not enable & ready
  for (i=0;i<8;i++)
    g_usbs_env.EP0.setup_data.PACKED[i] = 0;
}

void set_stall_flag() {
  g_usbs_env.EP0.stall_flag = 1;
}

void reset_stall_flag() {
  g_usbs_env.EP0.stall_flag = 0;
}

int get_stall_flag() {
  return g_usbs_env.EP0.stall_flag;
}

int check_ready_bit_ep(int en) {
  switch (en) {
  case 0:
    return (g_usbs_env.EP0.active & !(USB_ENDPOINT0_CONTROL_REG & 0x02));
  case 1:
    return (g_usbs_env.EP1.active & !(USB_ENDPOINT1_CONTROL_REG & 0x02));
  case 2:
    return (g_usbs_env.EP2.active & !(USB_ENDPOINT2_CONTROL_REG & 0x02));
  case 3:
    return (g_usbs_env.EP3.active & !(USB_ENDPOINT3_CONTROL_REG & 0x02));
  default:
    return 0;
  }
}

void trans_done_event_proc() {
  if (check_ready_bit_ep(0)) endpoint0_trans();
   if ( g_usbs_env.usb_device_state == UDS_CONFIGURED) {
    if (check_ready_bit_ep(1))  endpoint1_trans();
    if (check_ready_bit_ep(2))  endpoint2_trans();
   }
}

void nak_sent_event_proc() {
  if (g_usbs_env.usb_device_state == UDS_CONFIGURED) {
    if (g_usbs_env.EP0.control_cur_stage == UDC_IDLE) {
      USB_ENDPOINT1_CONTROL_REG = 0x1;
      USB_ENDPOINT2_CONTROL_REG = 0x3;
      USB_ENDPOINT3_CONTROL_REG = 0x1;
    }
  } else USB_ENDPOINT0_CONTROL_REG = 0x3;
}


int rx_fifo_cnt(int ep) {
  if (ep == 0) return (USB_EP0_RX_FIFO_DATA_COUNT_MSB << 8) | USB_EP0_RX_FIFO_DATA_COUNT_LSB;
  else
  if (ep == 1) return (USB_EP1_RX_FIFO_DATA_COUNT_MSB << 8) | USB_EP1_RX_FIFO_DATA_COUNT_LSB;
  else
  if (ep == 2) return (USB_EP2_RX_FIFO_DATA_COUNT_MSB << 8) | USB_EP2_RX_FIFO_DATA_COUNT_LSB;
  else
  if (ep == 3) return (USB_EP3_RX_FIFO_DATA_COUNT_MSB << 8) | USB_EP3_RX_FIFO_DATA_COUNT_LSB;
  else return -1;
}
