//=======================================================================
//                         USB Target CDC sample
//
// File:
//   usbs_desc.c
//
// Abstract:
//   descriptor tables
//
//  Created:
//    27 November 2014
//======================================================================
//
// Copyright (c) 2014, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Revision History
// $Date: 2015-01-08 07:25:52 +0900 (Thu, 08 Jan 2015) $
// $Rev: 12 $

#include "usbs_struct.h"

// device descriptor
// p.262
unsigned char g_usb_device_desc[] = {
  18,  //bLength
  UDT_DEVICE, // bDescriptorType
  0x10,0x01,  // USB Version 1.10
  0x02,       // bDeviceClass(defined by the interface descriptors)
  0x00,       // bDeviceSubClass;
  0x00,       // bDeviceProtocol;
  0x08,       // bMaxPacketSize0: max packet size for endpoint0 = 8Bytes; for full-speed
  0xff,0x26,  // idVendor[2]: 0x26ff
  0x12,0x49,  // idProduct[2]: 0x4912
  0x00,0x01,  // bcdDevice[2]: 0x0100
  0x1,        // iManufacturer;
  0x2,        // iProduct;
  0x3,        // iSerialNumber;
  0x1         // bNumConfigurations
};

// configuration descriptor
// p.265
unsigned char g_usb_conf_desc[] = {
  9,          // bLength
  UDT_CONFIGURATION, // bDescriptorType
  0x43,0x00,  // 67bytes,wTotalLength[2];
  2,          // bNumInterfaces;
  1,          // bConfigurationValue;
  0,          //iConfiguration;
  0xc0,       //bmAttributes;
  250,        // 500mA bMaxPower;
  /*************** Interface0 ***********************/
  9,          // bLength
  UDT_INTERFACE, //  bDescriptorType
  0,          // bInterfaceNumber
  0,          // bAlernateSetting
  1,          // bNumEndpoints
  0x2,        // Communication Interface Class :bInterfaceClass
  0x2,        // Abstract Control : bInterfaceSubClass
  0x1,        // AT Command : bInterfaceProtocol
  0,          // iInterface
  // Header Functional Descriptor
  5,          // bFunctionLength
  0x24,       // bDescriptorType
  0,          // bDescriptorSubType
  0x10,0x01,  // bcdCDC (CDC version 1.1)
  // Abstract Control Management Functional Descriptor
  4,          // bFunctionLength
  0x24,       // bDescriptorType
  0x02,       // bDescriptorSubType
  0x02,       // bmCapabilities (Set/Get LineCoding support)
  // Union Interface Function Descriptor
  5,          // bFunctionLength
  0x24,       // bDescriptorType
  0x6,        // bDescriptorSubType
  0x00,       // bMasterInterface0 (Interface#0 is control class)
  0x01,       // bSlaveInterface0 (Interface#1 is slave(Data class))
  // CallManagement Function Descriptor
  5,          // bFunctionLength
  0x24,       // bDescriptorType
  0x01,       // bDescriptorSubType
  0x00,       // bCapability
  0x01,       // bDataInterface
  // Endpoint Descriptor 3
  0x7,        // bLength
  UDT_ENDPOINT, // bDescriptorType
  0x83,       // bEndpointAddress IN,3
  0x03,       // bmAttributes: Interrupt
  0x08,0x00,  // wMaxPacketSize
  10,         // bInterval: 10ms
  /*************** interface descriptor1 ******************/
  9,          // bLength
  UDT_INTERFACE, //  bDescriptorType
  1,          //  bInterfaceNumber
  0,          //  bAlernateSetting
  2,          //  bNumEndpoints
  0x0a,       // Data Interface Class :bInterfaceClass
  0x00,       // bInterfaceSubClass
  0x00,       // bInterfaceProtocol
  0,          // iInterface
  // Endpoint Descriptor 1
  0x7,        // bLength;
  UDT_ENDPOINT,// bDescriptorType
  0x01,       // bEndpointAddress OUT,1
  0x02,       // bmAttributes: Bulk
  0x40,0x00,  // wMaxPacketSize:64
  00,         // bInterval: 10ms
  // Endpoint Descriptor 2
  0x7,        // bLength
  UDT_ENDPOINT,// bDescriptorType
  0x82,       // bEndpointAddress IN,2
  0x02,       // bmAttributes: Bulk
  0x40,0x00,  // wMaxPacketSize:64
  00          // bInterval: 10ms
};


// String Descriptors
//   Descriptor Zero
unsigned char g_usb_string_desc0[] = {
  0x4,         // bLength
  UDT_STRING,  //  bDescriptorType
  0x09,0x04    //  English(United States)
};

// iManufacturer
unsigned char g_usb_string_desc1[] = {
  42,          // bLength = String length + 2bytes
  UDT_STRING,  //  bDescriptorType
  'O',0x00,
  'p',0x00,
  'e',0x00,
  'n',0x00,
  'C',0x00,
  'o',0x00,
  'r',0x00,
  's',0x00,
  ' ',0x00,
  'U',0x00,
  'S',0x00,
  'B',0x00,
  ' ',0x00,
  'D',0x00,
  'e',0x00,
  'v',0x00,
  'i',0x00,
  'c',0x00,
  'e',0x00,
  '.',0x00
};

// idProduct
unsigned char g_usb_string_desc2[] = {
  22,          // bLength
  UDT_STRING,  //  bDescriptorType
  'U',0x00,
  'S',0x00,
  'B',0x00,
  ' ',0x00,
  'S',0x00,
  'e',0x00,
  'r',0x00,
  'i',0x00,
  'a',0x00,
  'l',0x00
};

// iSerialNumber
unsigned char g_usb_string_desc3[] = {
  20,           // bLength;
  UDT_STRING,   //  bDescriptorType;
  '1',0x00,
  '2',0x00,
  '3',0x00,
  '4',0x00,
  ' ',0x00,
  'A',0x00,
  'B',0x00,
  'C',0x00,
  'D',0x00
};
