//=======================================================================
//                         USB Target CDC sample
//
// File:
//   usbs_struct.h
//
// Abstract:
//   state structs
//
//  Created:
//    27 November 2014
//======================================================================
//
// Copyright (c) 2014, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Revision History
// $Date: 2015-01-08 07:25:52 +0900 (Thu, 08 Jan 2015) $
// $Rev: 12 $
                  
#ifndef __USBS_STRUCT_H__
#define __USBS_STRUCT_H__

/* Device State */
typedef enum {
  UDS_DEFAULT,   // device is attached, powered, and had been reset.
  UDS_ADDRESS,   // a unique device address had been assigned.
  UDS_CONFIGURED // device is configured, and is not suspended.
} en_usbs_device_state;


/* Control Transfer Stage */
/* current stage */
typedef enum {
  UDC_IDLE,
  UDC_SETUP_READ,
  UDC_SETUP_WRITE,
  UDC_SETUP_NO_DATA,
  UDC_DATA_READ,
  UDC_DATA_WRITE,
  UDC_STATUS_READ,
  UDC_STATUS_WRITE,
  UDC_STATUS_NO_DATA,
} en_usbs_control_transfer_stage;


/* Standard Request Codes */
typedef enum {
  USR_GET_STATUS = 0,
  USR_CLEAR_FEATURE = 1,
  USR_SET_ADDRESS = 5,
  USR_GET_DESCRIPTOR = 6,
  USR_SET_DESCRIPTOR = 7,
  USR_GET_CONFIGURATION = 8,
  USR_SET_CONFIGURATION = 9,
  USR_GET_INTERFACE = 10,
  USR_SET_INTERFACE = 11,
  USR_SYNCH_FRAME = 12,
} en_usb_std_request_code;

typedef enum {
  USR_SET_LINE_CODING = 0x20,
  USR_GET_LINE_CODING = 0x21,
  USR_SET_CONTROL_LINE_STATE = 0x22
} en_usb_class_request_code;

/* descriptor type */
typedef enum {
  UDT_DEVICE = 1,
  UDT_CONFIGURATION = 2,
  UDT_STRING = 3,
  UDT_INTERFACE = 4,
  UDT_ENDPOINT = 5,
} en_usb_descriptor_type;

typedef enum {
  UDRT_STANDARD = 0,
  UDRT_CLASS = 1,
  UDRT_VENDOR = 2,
  UDRT_RESERVED = 3
} en_usb_device_request_type;

typedef enum {
  UDRR_DEVICE = 0,
  UDRR_INTERFACE = 1,
  UDRR_ENDPOINT = 2,
  UDRR_OTHER = 3
} en_usb_device_request_recipient;

typedef union {
  unsigned char PACKED[8];
  struct {
    unsigned char bmRequestType;
    unsigned char bRequest;
    unsigned char wValue[2];
    unsigned char wIndex[2];
    unsigned char wLength[2];
  } BYTE;
} un_usb_setup_data;

typedef struct {
  int active;
  int stall_flag;

  un_usb_setup_data setup_data;
  // control transfer current stage
  en_usbs_control_transfer_stage control_cur_stage;
  // control transfer previous stage for NAK
  en_usbs_control_transfer_stage control_prev_stage;

  int cur_desc_cnt;        // desc count
  int total_desc_cnt;
  int cur_byte_cnt;        // byte count
  int out_data_type;
} st_usbs_endpoint0;

typedef struct {
  int active;
  int out_data_type;
} st_usbs_endpoint;

typedef struct {
  en_usbs_device_state usb_device_state;
  st_usbs_endpoint0 EP0;
  st_usbs_endpoint  EP1;
  st_usbs_endpoint  EP2;
  st_usbs_endpoint  EP3;
} st_usbs_env;

#endif
